# -*- coding: utf-8 -*-
## PloneiCalendar
## Store, share, and browse your iCalendar files in Plone
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


from AccessControl import ClassSecurityInfo, Unauthorized
#from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFDefault.File import File
from Products.CMFCore.utils import getToolByName


import os
import datetime
import string
from StringIO import StringIO


# WebDAV support (in order to refresh the calendar views after a WebDAV upload)
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface


# Archetypes imports
from Products.Archetypes.public import *
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *


#Product imports
from Products.PloneiCalendar.parser import *
from Products.PloneiCalendar.config import *    
from schemata import iCalendarSchema
from actions import iCalendarActions

class iCalendar(BaseContent):
    """An PloneiCalendar is a content type with an ics file attached."""
    
    __implements__ = (
        BaseContent.__implements__ , WriteLockInterface,
    )
    
    # Standard security settings
    security = ClassSecurityInfo()
    
    meta_type      = 'iCalendar'
    archetype_name = 'iCalendar'

    content_icon   = 'PloneiCalendar.gif'
    immediate_view = 'PloneiCalendar_editForm'
    
    global_allow   = 1
    include_default_actions = 0
    schema         = iCalendarSchema
    actions        = iCalendarActions
    
    
    security.declarePrivate('initializeLocalRoles')
    def initializeLocalRoles(self):
        """Initialize local roles of calendar"""
    
        # Set local roles LocalWriter and LocalOwner for owner user
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        member_id = member.getUserName()
        roles = list(self.get_local_roles_for_userid( userid=member_id ))
        if "LocalWriter" not in roles and "LocalReader" not in roles :
            roles.append( "LocalWriter" )
            roles.append( "LocalReader" )
            self.manage_setLocalRoles( member_id, roles )
    
    security.declarePrivate('initializeFile')
    def initializeFile(self,):
        """Initialize file to an empty file"""
        
        # Set file field to a default calendar file
        filename = "calendar.ics"
        path = os.path.join(INSTANCE_HOME, 'Products', 'PloneiCalendar', filename)
        if not os.path.exists(path):
            return 
        
        try:            
            calendar_file = open(path)
            self.setFile(calendar_file.read(), filename=filename, mimetype='text/calendar')
        finally:
            calendar_file.close()
    
    security.declareProtected(CMFCorePermissions.View, 'index_html')
    def index_html(self, REQUEST, RESPONSE):
        """Download the file
        """
        
        field = self.getPrimaryField()
        return field.download(self)
        
    security.declareProtected(CMFCorePermissions.View, 'download')
    def download(self, REQUEST=None, RESPONSE=None):
        """Download the file (use default index_html)
        """
        if REQUEST is None:
            REQUEST = self.REQUEST
        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE
        field = self.getPrimaryField()
        return field.download(self, REQUEST, RESPONSE)
    
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        BaseContent.manage_afterAdd(self, item, container)
        
        # Initialize local roles
        self.initializeLocalRoles()
        
        # Initialize an empty calendar
        self.initializeFile()
        
        # New in version 1.0
        self._v_tree=None # the volatile list of VEvents
        
   
    security.declareProtected(CMFCorePermissions.View, "getICSUrl")
    def getICSUrl(self,):
        """ Return an url to access the ics file """
    
        return self.ics_url()
    
    security.declarePrivate('_parse')
    def _parse(self):
        """ Parses the attached ICS file and fills _v_tree with the corresponding
        events (VEvent objects). Raises an exception if the parser fails. """
        
        datas = getToolByName(self, 'portal_ploneicalendar').convertStringToPortalCharset(self, self.file.data)
        icr = ICalReader(StringIO(datas))
        lvevent = list(icr.iterEvents())
        lvevent.sort()
        dvevent = dict([(ev.getUID(),ev) for ev in lvevent if ev.getUID()])
        self._v_tree=lvevent
        self._v_rd=dvevent

    
    security.declarePrivate('accessVolatile')
    def accessVolatile(self):
        """ Verify that the volatile datas are still here before accessing it. If not, parse again the ics file """
        if not (hasattr(self,"_v_tree") and hasattr(self,"_v_rd")):
            self._parse()

    security.declareProtected(CMFCorePermissions.View, "listEvents")
    def listEvents(self,date1=datetime.datetime.today().replace(hour=8,minute=0,second=0), date2=datetime.datetime.today().replace(hour=22,minute=0,second=0), onlyFullEvents=0):
        """ Updates _v_tree if necessary, and then returns a sorted list
        (chronological order) with this object's events (VEvent objects) that
        occur between date1 and date2, including recurrent ones. """
        self.accessVolatile()
        lvev=[]
        for vev in self._v_tree:
            lvev=lvev+vev.getOccurrencesBetween(date1,date2,onlyFullEvents)
            lvev.sort()
        return lvev

    security.declareProtected(CMFCorePermissions.View, "getVEventByUID")
    def getVEventByUID(self, uid):
        """ Return the VEvent with the corresponding uid or None if it doesn't exist """
        self.accessVolatile()
        return self._v_rd.get(uid,None)

    # Functions that should definitively be moved into a tool or a set of PythonScripts
    # accompanying the product.
    
    security.declareProtected(CMFCorePermissions.View, "listAllCalendars")
    def listAllCalendars(self):
        """ Return a list of all PloneiCalendar objects in this portal. """
        return getToolByName(self, 'portal_ploneicalendar').listAllCalendars(self)
      
    security.declareProtected(CMFCorePermissions.View, "listCalendars")
    def listCalendars(self, user=None, readable=0, writeable=0):
        """ Return a list of all PloneiCalendar objects in this portal readable
          or writeable by the user (as specified). If a parameter equals 0, it is
          ignored. If the user is not given, assume that it is the current one. """

        return getToolByName(self, 'portal_ploneicalendar').listCalendars(self, user, readable, writeable)
        
class PloneiCalendar(iCalendar):
    pass
    # Define to make backward compatibility with PloneiCalendar 1.1

registerType(iCalendar, PROJECTNAME)
